#pragma once

#include "Vec3.h"
#include <cmath>

class Mat3
{
public:
	Mat3() = default;
	Mat3( float m00, float m01, float m02,
		  float m10, float m11, float m12,
		  float m20, float m21, float m22 )
	{
		r[ 0 ].x = m00;
		r[ 0 ].y = m01;
		r[ 0 ].z = m02;

		r[ 1 ].x = m10;
		r[ 1 ].y = m11;
		r[ 1 ].z = m12;

		r[ 2 ].x = m20;
		r[ 2 ].y = m21;
		r[ 2 ].z = m22;
	}
	Mat3( const Vec3 &R0, const Vec3 &R1, const Vec3 &R2 )
	{
		r[ 0 ] = R0;
		r[ 1 ] = R1;
		r[ 2 ] = R2;
	}

	Mat3 Transpose()const
	{
		return Mat3( *this ).Transpose();
	}
	Mat3 &Transpose()
	{
		return Mat3{
			{ r[ 0 ].x, r[ 1 ].x, r[ 2 ].x },
			{ r[ 0 ].y, r[ 1 ].y, r[ 2 ].y },
			{ r[ 0 ].z, r[ 1 ].z, r[ 2 ].z }
		};
	}

	static Mat3 Identity()
	{
		return{
			{ 1.f, 0.f, 0.f }, 
			{ 0.f, 1.f, 0.f },
			{ 0.f, 0.f, 1.f }
		};
	}
	static Mat3 Translation( const Vec2 &Offset )
	{
		Mat3 result = Identity();

		result.r[ 2 ].x = Offset.x;
		result.r[ 2 ].y = Offset.y;

		return result;
	}
	static Mat3 Scale( const float X, const float Y )
	{
		Mat3 result = Identity();

		result.r[ 0 ].x = X;
		result.r[ 1 ].y = Y;

		return result;
	}
	static Mat3 Rotate( const float RadianAngle )
	{
		const float sine = std::sinf( RadianAngle );
		const float cosine = std::cosf( RadianAngle );

		Mat3 result = Identity();

		result.r[ 0 ].x = cosine; result.r[ 0 ].y = -sine;
		result.r[ 1 ].x = sine;   result.r[ 1 ].y = cosine;

		return result;
	}

	Mat3 operator*( const Mat3 &Src )const
	{
		Mat3 temp = Src.Transpose();
		Mat3 result;
		for( int i = 0; i < 3; ++i )
		{
			result.r[ i ].x = r[ i ].DotProduct( temp.r[ 0 ] );
			result.r[ i ].y = r[ i ].DotProduct( temp.r[ 1 ] );
			result.r[ i ].z = r[ i ].DotProduct( temp.r[ 2 ] );
		}
		return result;
	}
	Vec2 operator*( const Vec2 &Src )const
	{
		Vec3 tempSrc = Vec3( Src );
		Mat3 trans = Transpose();
		Vec3 tempRslt;

		tempRslt.x = tempSrc.DotProduct( trans.r[ 0 ] );
		tempRslt.y = tempSrc.DotProduct( trans.r[ 1 ] );
		tempRslt.z = tempSrc.DotProduct( trans.r[ 2 ] );


		return Vec2( tempRslt.x, tempRslt.y );
	}
public:
	Vec3 r[ 3 ];
};

