#include "Ship.h"
#include "Mat3.h"

Ship::Ship(Vec2 pos_in, Color cr, Color cb)
	:
	pos(pos_in),
	cr(cr),
	cb(cb)
{
}

void Ship::Update(Mouse& mouse, Keyboard & kbd)
{
	if (kbd.KeyIsPressed(VK_UP))
	{
		pos.y -= speed;
	}
	else if (kbd.KeyIsPressed(VK_DOWN))
	{
		pos.y += speed;
	}
	if (kbd.KeyIsPressed(VK_LEFT))
	{
		pos.x -= speed;
	}
	else if (kbd.KeyIsPressed(VK_RIGHT))
	{
		pos.x += speed;
	}

	const auto delta = Vec2( 
		static_cast<float>( mouse.GetPosX() ), 
		static_cast<float>( mouse.GetPosY() ) ) - pos;

	dir = delta.Normalize();

}

void Ship::Draw(Graphics & gfx)
{
	const Vec2 v1 = { 0.f, 1.f };
	const Vec2 v2 = { -1.f, -1.f };
	const Vec2 v3 = { 1.f, -1.f };

	const Mat3 trans = Mat3::Translation( pos );
	const Mat3 scale = Mat3::Scale( dimension, dimension );

	const Mat3 rot = Mat3::Rotate( std::atan2f( dir.x, dir.y ) );

	const Mat3 transform = rot * scale * trans;

	const Vec2 p1 = transform * v1;
	const Vec2 p2 = transform * v2;
	const Vec2 p3 = transform * v3;

	gfx.DrawLine( p1.x, p1.y, p2.x, p2.y, cr );
	gfx.DrawLine( p1.x, p1.y, p3.x, p3.y, cr );
	gfx.DrawLine( p2.x, p2.y, p3.x, p3.y, cb );
}
