#pragma once

#include <cmath>

template<class T>
class _Vec2
{
public:
	_Vec2() = default;
	_Vec2( T x_in, T y_in)
		:
		x( x_in ),
		y( y_in )
	{
	}

	_Vec2 operator+(const _Vec2& rhs) const
	{
		return _Vec2( x + rhs.x, y + rhs.y );
	}
	_Vec2& operator+=(const _Vec2& rhs)
	{
		return *this = *this + rhs;
	}
	_Vec2 operator*( T rhs) const
	{
		return _Vec2( x * rhs, y * rhs );
	}
	_Vec2& operator*=( T rhs)
	{
		return *this = *this * rhs;
	}
	_Vec2 operator-(const _Vec2& rhs) const
	{
		return _Vec2( x - rhs.x, y - rhs.y );
	}
	_Vec2& operator-=(const _Vec2& rhs)
	{
		return *this = *this - rhs;
	}

	T GetLength() const
	{
		return std::sqrtf( GetLengthSq() );
	}
	T GetLengthSq() const
	{
		return x * x + y * y;
	}
	_Vec2& Normalize()
	{
		const float len = GetLength();
		*this *= len != 0.f ?
			1.f / len : 1.f;
		return *this;
	}
	_Vec2 Normalize() const
	{
		return _Vec2( *this ).Normalize();
	}

public:
	T x;
	T y;
};

using Vec2 = _Vec2<float>;